@@ -1,13 +1,13 @@
 /*
  * Copyright 2003-2012 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/pax-utils/scanelf.c,v 1.253 2012/11/30 23:25:07 vapier Exp $
+ * $Header: /var/cvsroot/gentoo-projects/pax-utils/scanelf.c,v 1.265 2014/03/21 05:33:33 vapier Exp $
  *
  * Copyright 2003-2012 Ned Ludd        - <solar@gentoo.org>
  * Copyright 2004-2012 Mike Frysinger  - <vapier@gentoo.org>
  */
 
-static const char rcsid[] = "$Id: scanelf.c,v 1.253 2012/11/30 23:25:07 vapier Exp $";
+static const char rcsid[] = "$Id: scanelf.c,v 1.265 2014/03/21 05:33:33 vapier Exp $";
 const char argv0[] = "scanelf";
 
 #include "paxinc.h"
@@ -18,7 +18,7 @@ const char argv0[] = "scanelf";
 static int file_matches_list(const char *filename, char **matchlist);
 
 /* variables to control behavior */
-static char *match_etypes = NULL;
+static array_t _match_etypes = array_init_decl, *match_etypes = &_match_etypes;
 static array_t _ldpaths = array_init_decl, *ldpaths = &_ldpaths;
 static char scan_ldpath = 0;
 static char scan_envpath = 0;
@@ -70,26 +70,67 @@ static void *ldcache = NULL;
 static size_t ldcache_size = 0;
 static unsigned long setpax = 0UL;
 
-static int has_objdump = 0;
+static const char *objdump;
 
-/* find the path to a file by name */
-static int bin_in_path(const char *fname)
+/* Find the path to a file by name.  Note: we do not currently handle the
+ * empty path element correctly (should behave by searching $PWD). */
+static const char *which(const char *fname, const char *envvar)
 {
-	char fullpath[__PAX_UTILS_PATH_MAX];
-	char *path, *p;
+	size_t path_len, fname_len;
+	const char *env_path;
+	char *path, *p, *ep;
 
-	path = getenv("PATH");
-	if (!path)
-		return 0;
+	p = getenv(envvar);
+	if (p)
+		return p;
 
-	while ((p = strrchr(path, ':')) != NULL) {
-		snprintf(fullpath, sizeof(fullpath), "%s/%s", p + 1, fname);
-		*p = 0;
-		if (access(fullpath, R_OK) != -1)
-			return 1;
+	env_path = getenv("PATH");
+	if (!env_path)
+		return NULL;
+
+	/* Create a copy of the $PATH that we can safely modify.
+	 * Make it a little bigger so we can append "/fname".
+	 * We do this twice -- once for a perm copy, and once for
+	 * room at the end of the last element. */
+	path_len = strlen(env_path);
+	fname_len = strlen(fname);
+	path = xmalloc(path_len + (fname_len * 2) + 2 + 2);
+	memcpy(path, env_path, path_len + 1);
+
+	p = path + path_len + 1 + fname_len + 1;
+	*p = '/';
+	memcpy(p + 1, fname, fname_len + 1);
+
+	/* Repoint fname to the copy in the env string as it has
+	 * the leading slash which we can include in a single memcpy.
+	 * Increase the fname len to include the '/' and '\0'. */
+	fname = p;
+	fname_len += 2;
+
+	p = path;
+	while (p) {
+		ep = strchr(p, ':');
+		/* Append the /foo path to the current element. */
+		if (ep)
+			memcpy(ep, fname, fname_len);
+		else
+			memcpy(path + path_len, fname, fname_len);
+
+		if (access(p, R_OK) != -1)
+			return p;
+
+		p = ep;
+		if (ep) {
+			/* If not the last element, restore the chunk we clobbered. */
+			size_t offset = ep - path;
+			size_t restore = min(path_len - offset, fname_len);
+			memcpy(ep, env_path + offset, restore);
+			++p;
+		}
 	}
 
-	return 0;
+	free(path);
+	return NULL;
 }
 
 static FILE *fopenat_r(int dir_fd, const char *path)
@@ -379,7 +420,11 @@ static char *scanelf_file_phdr(elfobj *elf, char *found_phdr, char *found_relro,
 		char *str; \
 		if ((void*)strtbl > elf->data_end) \
 			goto skip_this_shdr##B; \
-		check_flags = SHF_WRITE|SHF_EXECINSTR; \
+		/* let's flag -w/+x object files since the final ELF will most likely \
+		 * need write access to the stack (who doesn't !?).  so the combined \
+		 * output will bring in +w automatically and that's bad. \
+		 */ \
+		check_flags = /*SHF_WRITE|*/SHF_EXECINSTR; \
 		for (i = 0; i < EGET(ehdr->e_shnum); ++i) { \
 			if (EGET(shdr[i].sh_type) != SHT_PROGBITS) continue; \
 			offset = EGET(strtbl->sh_offset) + EGET(shdr[i].sh_name); \
@@ -566,23 +611,24 @@ static char *scanelf_file_textrels(elfobj *elf, char *found_textrels, char *foun
 			} else \
 				printf("(optimized out)"); \
 			printf(" [0x%lX]\n", (unsigned long)offset_tmp); \
-			if (be_verbose && has_objdump) { \
+			if (be_verbose && objdump) { \
 				Elf ## B ## _Addr end_addr = offset_tmp + EGET(func->st_size); \
 				char *sysbuf; \
 				size_t syslen; \
-				const char sysfmt[] = "objdump -r -R -d -w -l --start-address=0x%lX --stop-address=0x%lX %s | grep --color -i -C 3 '.*[[:space:]]%lX:[[:space:]]*R_.*'\n"; \
-				syslen = sizeof(sysfmt) + strlen(elf->filename) + 3 * sizeof(unsigned long) + 1; \
+				const char sysfmt[] = "%s -r -R -d -w -l --start-address=0x%lX --stop-address=0x%lX %s | grep --color -i -C 3 '.*[[:space:]]%lX:[[:space:]]*R_.*'\n"; \
+				syslen = sizeof(sysfmt) + strlen(objdump) + strlen(elf->filename) + 3 * sizeof(unsigned long) + 1; \
 				sysbuf = xmalloc(syslen); \
 				if (end_addr < r_offset) \
 					/* not uncommon when things are optimized out */ \
 					end_addr = r_offset + 0x100; \
 				snprintf(sysbuf, syslen, sysfmt, \
+					objdump, \
 					(unsigned long)offset_tmp, \
 					(unsigned long)end_addr, \
 					elf->filename, \
 					(unsigned long)r_offset); \
 				fflush(stdout); \
-				if (system(sysbuf)) /* don't care */; \
+				if (system(sysbuf)) {/* don't care */} \
 				fflush(stdout); \
 				free(sysbuf); \
 			} \
@@ -903,7 +949,7 @@ static char *lookup_cache_lib(elfobj *elf, const char *fname)
 }
 #endif
 
-static char *lookup_config_lib(elfobj *elf, char *fname)
+static char *lookup_config_lib(const char *fname)
 {
 	static char buf[__PAX_UTILS_PATH_MAX] = "";
 	const char *ldpath;
@@ -965,7 +1011,7 @@ static const char *scanelf_file_needed_lib(elfobj *elf, char *found_needed, char
 						if (!be_wewy_wewy_quiet) { \
 							if (*found_needed) xchrcat(ret, ',', ret_len); \
 							if (use_ldpath) { \
-								if ((p = lookup_config_lib(elf, needed)) != NULL) \
+								if ((p = lookup_config_lib(needed)) != NULL) \
 									needed = p; \
 							} else if (use_ldcache) { \
 								if ((p = lookup_cache_lib(elf, needed)) != NULL) \
@@ -1041,11 +1087,11 @@ static char *scanelf_file_interp(elfobj *elf, char *found_interp)
 
 	return NULL;
 }
-static char *scanelf_file_bind(elfobj *elf, char *found_bind)
+static const char *scanelf_file_bind(elfobj *elf, char *found_bind)
 {
 	unsigned long i;
 	struct stat s;
-	char dynamic = 0;
+	bool dynamic = false;
 
 	if (!show_bind) return NULL;
 	if (!elf->phdr) return NULL;
@@ -1058,7 +1104,7 @@ static char *scanelf_file_bind(elfobj *elf, char *found_bind)
 		Elf ## B ## _Off offset; \
 		for (i = 0; i < EGET(ehdr->e_phnum); i++) { \
 			if (EGET(phdr[i].p_type) != PT_DYNAMIC || EGET(phdr[i].p_filesz) == 0) continue; \
-			dynamic = 1; \
+			dynamic = true; \
 			offset = EGET(phdr[i].p_offset); \
 			if (offset >= elf->len - sizeof(Elf ## B ## _Dyn)) continue; \
 			dyn = DYN ## B (elf->vdata + offset); \
@@ -1084,7 +1130,7 @@ static char *scanelf_file_bind(elfobj *elf, char *found_bind)
 		return NULL;
 	} else {
 		*found_bind = 1;
-		return (char *)(dynamic ? "LAZY" : "STATIC");
+		return dynamic ? "LAZY" : "STATIC";
 	}
 }
 static char *scanelf_file_soname(elfobj *elf, char *found_soname)
@@ -1546,43 +1592,39 @@ static int scanelf_elfobj(elfobj *elf)
 static int scanelf_elf(const char *filename, int fd, size_t len)
 {
 	int ret = 1;
+	size_t n;
+	const char *match_etype;
 	elfobj *elf;
 
-	/* verify this is real ELF */
+	/* Verify this is a real ELF */
 	if ((elf = _readelf_fd(filename, fd, len, !fix_elf)) == NULL) {
 		if (be_verbose > 2) printf("%s: not an ELF\n", filename);
 		return 2;
 	}
+
+	/* Possibly filter based on ELF bitness */
 	switch (match_bits) {
-		case 32:
-			if (elf->elf_class != ELFCLASS32)
-				goto label_done;
-			break;
-		case 64:
-			if (elf->elf_class != ELFCLASS64)
-				goto label_done;
-			break;
-		default: break;
-	}
-	if (match_etypes) {
-		char sbuf[126];
-		strncpy(sbuf, match_etypes, sizeof(sbuf));
-		if (strchr(match_etypes, ',') != NULL) {
-			char *p;
-			while ((p = strrchr(sbuf, ',')) != NULL) {
-				*p = 0;
-				if (etype_lookup(p+1) == get_etype(elf))
-					goto label_ret;
-			}
-		}
-		if (etype_lookup(sbuf) != get_etype(elf))
-			goto label_done;
+	case 32:
+		if (elf->elf_class != ELFCLASS32)
+			goto done;
+		break;
+	case 64:
+		if (elf->elf_class != ELFCLASS64)
+			goto done;
+		break;
 	}
 
-label_ret:
+	/* Possibly filter based on the ELF's e_type field */
+	array_for_each(match_etypes, n, match_etype)
+		if (etype_lookup(match_etype) == get_etype(elf))
+			goto scanit;
+	if (array_cnt(match_etypes))
+		goto done;
+
+ scanit:
 	ret = scanelf_elfobj(elf);
 
-label_done:
+ done:
 	unreadelf(elf);
 	return ret;
 }
@@ -1692,7 +1734,8 @@ static int scanelf_dirat(int dir_fd, const char *path)
 	if (dir == NULL) {
 		if (subdir_fd != -1)
 			close(subdir_fd);
-		warnfp("could not opendir(%s)", path);
+		else if (be_verbose > 2)
+			printf("%s: skipping dir: %s\n", path, strerror(errno));
 		return 1;
 	}
 	if (be_verbose > 1) printf("%s: scanning dir\n", path);
@@ -1828,7 +1871,7 @@ static int _load_ld_cache_config(const char *fname)
 
 	if (curr_fd != -1) {
 		if (fchdir(curr_fd))
-			/* don't care */;
+			{/* don't care */}
 		close(curr_fd);
 	}
 
@@ -1892,12 +1935,10 @@ static void load_ld_cache_config(const char *fname)
 	_load_ld_cache_config(fname);
 
 	scan_l = scan_ul = scan_ull = false;
-	if (array_cnt(ldpaths)) {
-		array_for_each(ldpaths, n, ldpath) {
-			if (!scan_l   && !strcmp(ldpath, "/lib"))           scan_l   = true;
-			if (!scan_ul  && !strcmp(ldpath, "/usr/lib"))       scan_ul  = true;
-			if (!scan_ull && !strcmp(ldpath, "/usr/local/lib")) scan_ull = true;
-		}
+	array_for_each(ldpaths, n, ldpath) {
+		if (!scan_l   && !strcmp(ldpath, "/lib"))           scan_l   = true;
+		if (!scan_ul  && !strcmp(ldpath, "/usr/lib"))       scan_ul  = true;
+		if (!scan_ull && !strcmp(ldpath, "/usr/local/lib")) scan_ull = true;
 	}
 
 	if (!scan_l)   xarraypush_str(ldpaths, "/lib");
@@ -2029,10 +2070,23 @@ static const char * const opts_help[] = {
 /* display usage and exit */
 static void usage(int status)
 {
-	unsigned long i;
+	const char a_arg[] = "<arg>";
+	size_t a_arg_len = strlen(a_arg) + 2;
+	size_t i;
+	int optlen;
 	printf("* Scan ELF binaries for stuff\n\n"
 	       "Usage: %s [options] <dir1/file1> [dir2 dirN file2 fileN ...]\n\n", argv0);
 	printf("Options: -[%s]\n", PARSE_FLAGS);
+
+	/* prescan the --long opt length to auto-align */
+	optlen = 0;
+	for (i = 0; long_opts[i].name; ++i) {
+		int l = strlen(long_opts[i].name);
+		if (long_opts[i].has_arg == a_argument)
+			l += a_arg_len;
+		optlen = max(l, optlen);
+	}
+
 	for (i = 0; long_opts[i].name; ++i) {
 		/* first output the short flag if it has one */
 		if (long_opts[i].val > '~')
@@ -2042,9 +2096,10 @@ static void usage(int status)
 
 		/* then the long flag */
 		if (long_opts[i].has_arg == no_argument)
-			printf("--%-14s", long_opts[i].name);
+			printf("--%-*s", optlen, long_opts[i].name);
 		else
-			printf("--%-7s <arg> ", long_opts[i].name);
+			printf("--%s %s %*s", long_opts[i].name, a_arg,
+				(int)(optlen - strlen(long_opts[i].name) - a_arg_len), "");
 
 		/* finally the help text */
 		printf("* %s\n", opts_help[i]);
@@ -2063,6 +2118,16 @@ static void usage(int status)
 		flags &= ~PF_NO##flag; \
 		flags |= PF_##flag; \
 	}
+static void parse_delimited(array_t *arr, char *arg, const char *delim)
+{
+	char *ele = strtok(arg, delim);
+	if (!ele)	/* edge case: -s '' */
+		xarraypush_str(arr, "");
+	while (ele) {
+		xarraypush_str(arr, ele);
+		ele = strtok(NULL, delim);
+	}
+}
 static int parseargs(int argc, char *argv[])
 {
 	int i;
@@ -2086,7 +2151,8 @@ static int parseargs(int argc, char *argv[])
 			from_file = optarg;
 			break;
 		case 'E':
-			match_etypes = optarg;
+			/* historically, this was comma delimited */
+			parse_delimited(match_etypes, optarg, ",");
 			break;
 		case 'M':
 			match_bits = atoi(optarg);
@@ -2109,17 +2175,10 @@ static int parseargs(int argc, char *argv[])
 		case 'k':
 			xarraypush_str(find_section_arr, optarg);
 			break;
-		case 's': {
+		case 's':
 			/* historically, this was comma delimited */
-			char *this_sym = strtok(optarg, ",");
-			if (!this_sym)	/* edge case: -s '' */
-				xarraypush_str(find_sym_arr, "");
-			while (this_sym) {
-				xarraypush_str(find_sym_arr, this_sym);
-				this_sym = strtok(NULL, ",");
-			}
+			parse_delimited(find_sym_arr, optarg, ",");
 			break;
-		}
 		case 'N':
 			xarraypush_str(find_lib_arr, optarg);
 			break;
@@ -2192,8 +2251,8 @@ static int parseargs(int argc, char *argv[])
 		case 'b': show_bind = 1; break;
 		case 'S': show_soname = 1; break;
 		case 'T': show_textrels = 1; break;
-		case 'q': be_quiet = 1; break;
-		case 'v': be_verbose = (be_verbose % 20) + 1; break;
+		case 'q': be_quiet = min(be_quiet, 20) + 1; break;
+		case 'v': be_verbose = min(be_verbose, 20) + 1; break;
 		case 'a': show_perms = show_pax = show_phdr = show_textrel = show_rpath = show_bind = show_endian = 1; break;
 		case 'D': show_endian = 1; break;
 		case 'I': show_osabi = 1; break;
@@ -2213,7 +2272,7 @@ static int parseargs(int argc, char *argv[])
 		}
 	}
 	if (show_textrels && be_verbose)
-		has_objdump = bin_in_path("objdump");
+		objdump = which("objdump", "OBJDUMP");
 	/* precompile all the regexes */
 	if (g_match) {
 		regex_t preg;
@@ -2249,12 +2308,9 @@ static int parseargs(int argc, char *argv[])
 		}
 	}
 	/* flatten arrays for display */
-	if (array_cnt(find_sym_arr))
-		find_sym = array_flatten_str(find_sym_arr);
-	if (array_cnt(find_lib_arr))
-		find_lib = array_flatten_str(find_lib_arr);
-	if (array_cnt(find_section_arr))
-		find_section = array_flatten_str(find_section_arr);
+	find_sym = array_flatten_str(find_sym_arr);
+	find_lib = array_flatten_str(find_lib_arr);
+	find_section = array_flatten_str(find_section_arr);
 	/* let the format option override all other options */
 	if (out_format) {
 		show_pax = show_phdr = show_textrel = show_rpath = \
