@@ -1,7 +1,7 @@
 /*
  * Copyright 2003-2012 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/pax-utils/xfuncs.h,v 1.7 2012/11/04 07:26:24 vapier Exp $
+ * $Header: /var/cvsroot/gentoo-projects/pax-utils/xfuncs.h,v 1.8 2014/03/20 07:59:27 vapier Exp $
  *
  * Copyright 2003-2012 Ned Ludd        - <solar@gentoo.org>
  * Copyright 2004-2012 Mike Frysinger  - <vapier@gentoo.org>
@@ -29,7 +29,9 @@ void xarraypush(array_t *array, const void *ele, size_t ele_len);
 void xarrayfree(array_t *array);
 #define xrealloc_array(ptr, size, ele_size) xrealloc(ptr, (size) * (ele_size))
 #define array_for_each(arr, n, ele) \
-	for (n = 0, ele = arr->eles[n]; n < arr->num; ++n, ele = arr->eles[n])
+	for (n = 0, ele = array_cnt(arr) ? arr->eles[n] : NULL; \
+	     n < array_cnt(arr); \
+	     ele = arr->eles[++n])
 #define array_init_decl { .eles = NULL, .num = 0, }
 #define array_cnt(arr) (arr)->num
 char *array_flatten_str(array_t *array);
